# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_Clef___copy___01():
    r'''Copies explicit clefs copy.
    '''

    staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8 fs'8 g'8")
    clef = Clef('treble')
    attach(clef, staff[0])
    clef = Clef('bass')
    attach(clef, staff[4])
    copied_notes = mutate(staff[:2]).copy()
    staff.extend(copied_notes)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \clef "treble"
            c'8
            cs'8
            d'8
            ef'8
            \clef "bass"
            e'8
            f'8
            fs'8
            g'8
            \clef "treble"
            c'8
            cs'8
        }
        ''')

    assert inspect_(staff).is_well_formed()
    assert inspect_(staff[0]).get_effective(Clef) == Clef('treble')
    assert inspect_(staff[1]).get_effective(Clef) == Clef('treble')
    assert inspect_(staff[2]).get_effective(Clef) == Clef('treble')
    assert inspect_(staff[3]).get_effective(Clef) == Clef('treble')
    assert inspect_(staff[4]).get_effective(Clef) == Clef('bass')
    assert inspect_(staff[5]).get_effective(Clef) == Clef('bass')
    assert inspect_(staff[6]).get_effective(Clef) == Clef('bass')
    assert inspect_(staff[7]).get_effective(Clef) == Clef('bass')
    assert inspect_(staff[8]).get_effective(Clef) == Clef('treble')
    assert inspect_(staff[9]).get_effective(Clef) == Clef('treble')


def test_indicatortools_Clef___copy___02():
    r'''Does not copy implicit clefs.
    '''

    staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8 fs'8 g'8")
    clef = Clef('treble')
    attach(clef, staff[0])
    clef = Clef('bass')
    attach(clef, staff[4])
    copied_notes = mutate(staff[2:4]).copy()
    staff.extend(copied_notes)

    assert inspect_(staff).is_well_formed()
    assert inspect_(staff[0]).get_effective(Clef) == Clef('treble')
    assert inspect_(staff[1]).get_effective(Clef) == Clef('treble')
    assert inspect_(staff[2]).get_effective(Clef) == Clef('treble')
    assert inspect_(staff[3]).get_effective(Clef) == Clef('treble')
    assert inspect_(staff[4]).get_effective(Clef) == Clef('bass')
    assert inspect_(staff[5]).get_effective(Clef) == Clef('bass')
    assert inspect_(staff[6]).get_effective(Clef) == Clef('bass')
    assert inspect_(staff[7]).get_effective(Clef) == Clef('bass')
    assert inspect_(staff[8]).get_effective(Clef) == Clef('bass')
    assert inspect_(staff[9]).get_effective(Clef) == Clef('bass')

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \clef "treble"
            c'8
            cs'8
            d'8
            ef'8
            \clef "bass"
            e'8
            f'8
            fs'8
            g'8
            d'8
            ef'8
        }
        ''')



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedPitch___sub___01():

    pitch = NamedPitch(12)
    named_interval = pitchtools.NamedInterval('diminished', 3)

    assert pitch - named_interval == NamedPitch('as', 4)


def test_pitchtools_NamedPitch___sub___02():

    pitch = NamedPitch(12)
    numbered_interval = pitchtools.NumberedInterval(2)

    assert pitch - numbered_interval == NamedPitch('bf', 4)


def test_pitchtools_NamedPitch___sub___03():

    pitch_1 = NamedPitch(12)
    pitch_2 = NamedPitch(10)

    assert pitch_1 - pitch_2 == pitchtools.NamedInterval('major', -2)
    assert pitch_2 - pitch_1 == pitchtools.NamedInterval('major', 2)



# -*- coding: utf-8 -*-
from abjad import *


def test_systemtools_WellformednessManager_check_beamed_quarter_notes_01():
    r'''Beamed quarter notes are not well-formed.

    Here the beam attaches directly to the quarter notes.
    '''

    staff = Staff("c'4 d'4 e'4 f'4")
    beam = Beam()
    attach(beam, staff[:])

    assert not inspect_(staff).is_well_formed()


# -*- coding: utf-8 -*-
import collections


def cumulative_signed_weights(argument):
    r'''Gets cumulative signed weights of `argument`.

    ..  container:: example

        ::

            >>> argument = [1, -2, -3, 4, -5, -6, 7, -8, -9, 10]
            >>> mathtools.cumulative_signed_weights(argument)
            [1, -3, -6, 10, -15, -21, 28, -36, -45, 55]

        ::

            >>> argument = [-1, -2, -3, -4, -5, 0, 0, 0, 0, 0]
            >>> mathtools.cumulative_signed_weights(argument)
            [-1, -3, -6, -10, -15, -15, -15, -15, -15, -15]

        ::

            >>> argument = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
            >>> mathtools.cumulative_signed_weights(argument)
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]

    Raises type error when `argument` is not iterable.

    Returns new object of `argument` type.
    '''
    from abjad.tools import mathtools
    if not isinstance(argument, collections.Iterable):
        raise TypeError(argument)
    result = []
    for item in argument:
        try:
            next_element = abs(previous) + abs(item)
            previous_sign = mathtools.sign(previous)
        except NameError:
            next_element = abs(item)
            previous_sign = 0
        sign = mathtools.sign(item)
        if sign == -1:
            next_element *= sign
        elif sign == 0:
            next_element *= previous_sign
        result.append(next_element)
        previous = next_element
    return result



# -*- coding: utf-8 -*-
from abjad import *


def test_templatetools_TwoStaffPianoScoreTemplate___call___01():

    template = templatetools.TwoStaffPianoScoreTemplate()
    score = template()

    r'''
    \context Score = "Two-Staff Piano Score" <<
        \context PianoStaff = "Piano Staff" <<
            \set PianoStaff.instrumentName = \markup { Piano }
            \set PianoStaff.shortInstrumentName = \markup { Pf. }
            \context Staff = "RH Staff" {
                \clef "treble"
                \context Voice = "RH Voice" {
                }
            }
            \context Staff = "LH Staff" {
                \clef "bass"
                \context Voice = "LH Voice" {
                }
            }
        >>
    >>
    '''

    assert format(score) == stringtools.normalize(
        r'''
        \context Score = "Two-Staff Piano Score" <<
            \context PianoStaff = "Piano Staff" <<
                \set PianoStaff.instrumentName = \markup { Piano }
                \set PianoStaff.shortInstrumentName = \markup { Pf. }
                \context Staff = "RH Staff" {
                    \clef "treble"
                    \context Voice = "RH Voice" {
                    }
                }
                \context Staff = "LH Staff" {
                    \clef "bass"
                    \context Voice = "LH Voice" {
                    }
                }
            >>
        >>
        '''
        )



# -*- coding: utf-8 -*-
import numbers


def is_negative_integer(argument):
    '''Is true when `argument` equals a negative integer. Otherwise false.

    ..  container:: example

        ::

            >>> mathtools.is_negative_integer(-1)
            True

        ::

            >>> mathtools.is_negative_integer(0)
            False

        ::

            >>> mathtools.is_negative_integer(99)
            False

    Returns true or false.
    '''
    if isinstance(argument, numbers.Number):
        if argument == int(argument):
            if argument < 0:
                return True
    return False



# -*- coding: utf-8 -*-
from abjad import *


def test_rhythmtreetools_RhythmTreeContainer___iter___01():

    leaf_a = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=3)
    leaf_b = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=2)
    leaf_c = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=1)

    container = rhythmtreetools.RhythmTreeContainer(
        preprolated_duration=1, children=[leaf_a, leaf_b, leaf_c])

    assert [x for x in container] == [leaf_a, leaf_b, leaf_c]



# -*- coding: utf-8 -*-
from abjad.tools import systemtools

systemtools.ImportManager.import_structured_package(
    __path__[0],
    globals(),
    )

_documentation_section = 'demos'

if 'main' in dir():
    del(globals()['main'])



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_spannertools_TextSpanner_format_01():

    staff = scoretools.Staff("c'4 d'4 e'4 f'4")
    markup_one = markuptools.Markup('one')
    markup_two = markuptools.Markup('two')
    markup_three = markuptools.Markup('three')
    line_segment = indicatortools.LineSegment()
    arrow = indicatortools.Arrow()
    text_spanner = spannertools.TextSpanner()
    attach(markup_one, staff[0], is_annotation=True)
    attach(line_segment, staff[0])
    attach(markup_two, staff[1], is_annotation=True)
    attach(arrow, staff[1])
    attach(markup_three, staff[-1], is_annotation=True)
    attach(text_spanner, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \once \override TextSpanner.bound-details.left.text = \markup { one }
            c'4 \startTextSpan
            \once \override TextSpanner.arrow-width = 0.25
            \once \override TextSpanner.bound-details.left-broken.text = ##f
            \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
            \once \override TextSpanner.bound-details.left.text = \markup {
                \concat
                    {
                        two
                        \hspace
                            #0.25
                    }
                }
            \once \override TextSpanner.bound-details.right-broken.padding = 0
            \once \override TextSpanner.bound-details.right.arrow = ##t
            \once \override TextSpanner.bound-details.right.padding = 1.5
            \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
            \once \override TextSpanner.dash-fraction = 1
            d'4 \stopTextSpan \startTextSpan
            e'4
            f'4 \stopTextSpan ^ \markup { three }
        }
        ''')


def test_spannertools_TextSpanner_format_02():
    r'''Regression test: makes sure text spanner ignores nonannotated markup;
    also makes sure nonannotated markup formats and appears in LilyPond
    output.
    '''

    staff = scoretools.Staff("c'4 d'4 e'4 f'4")
    nonannotated_markup = Markup('leggieriss.')
    attach(nonannotated_markup, staff[0])

    arrow_start = markuptools.Markup('ord.').upright()
    arrow_stop = markuptools.Markup('pont.').upright()
    markup_three = markuptools.Markup('three')
    arrow = indicatortools.Arrow()
    text_spanner = spannertools.TextSpanner()
    attach(arrow_start, staff[0], is_annotation=True)
    attach(arrow, staff[0])
    attach(arrow_stop, staff[-1], is_annotation=True)
    attach(text_spanner, staff[:])

    assert format(staff) == systemtools.TestManager.clean_string(
        r'''
        \new Staff {
            \once \override TextSpanner.arrow-width = 0.25
            \once \override TextSpanner.bound-details.left-broken.text = ##f
            \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
            \once \override TextSpanner.bound-details.left.text = \markup {
                \concat
                    {
                        \upright
                            ord.
                        \hspace
                            #0.25
                    }
                }
            \once \override TextSpanner.bound-details.right-broken.padding = 0
            \once \override TextSpanner.bound-details.right.arrow = ##t
            \once \override TextSpanner.bound-details.right.padding = 1.5
            \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
            \once \override TextSpanner.dash-fraction = 1
            c'4 \startTextSpan - \markup { leggieriss. }
            d'4
            e'4
            f'4 \stopTextSpan ^ \markup {
                \upright
                    pont.
                }
        }
        ''')


def test_spannertools_TextSpanner_format_03():

    staff = scoretools.Staff("c'4 d'4 e'4 f'4")
    markup_one = markuptools.Markup('one')
    markup_two = markuptools.Markup('two')
    markup_three = markuptools.Markup('three')
    text_spanner = spannertools.TextSpanner()
    attach(markup_one, staff[0], is_annotation=True)
    attach(markup_two, staff[1], is_annotation=True)
    attach(markup_three, staff[-1], is_annotation=True)
    attach(text_spanner, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'4 ^ \markup { one }
            d'4 ^ \markup { two }
            e'4
            f'4 ^ \markup { three }
        }
        ''')



# -*- coding: utf-8 -*-
import copy
import functools
import numbers
from abjad.tools import mathtools
from abjad.tools.pitchtools.Interval import Interval


@functools.total_ordering
class NumberedInterval(Interval):
    '''Numbered interval.

    ..  container:: example

        Initializes from number of semitones:

        ::

            >>> numbered_interval = NumberedInterval(-14)
            >>> numbered_interval
            NumberedInterval(-14)

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_number',
        )

    ### INITIALIZER ###

    def __init__(self, argument=None):
        from abjad.tools import pitchtools
        if isinstance(argument, (int, float, int)):
            number = argument
        elif isinstance(argument, pitchtools.Interval):
            number = argument.semitones
        elif isinstance(argument, pitchtools.IntervalClass):
            interval_class = pitchtools.NumberedIntervalClass(argument)
            number = interval_class.number
        elif argument is None:
            number = 0
        else:
            message = 'can not initialize {}: {!r}.'
            message = message.format(type(self).__name__, argument)
            raise TypeError(message)
        self._number = mathtools.integer_equivalent_number_to_integer(number)

    ### SPECIAL METHODS ###

    def __abs__(self):
        r'''Absolute value of numbered interval.

        Returns new numbered interval.
        '''
        return type(self)(abs(self._number))

    def __add__(self, argument):
        r'''Adds `argument` to numbered interval.

        Returns new numbered interval.
        '''
        if isinstance(argument, type(self)):
            number = self.number + argument.number
            return type(self)(number)
        message = 'must be {}: {!r}.'
        message = message.format(type(self), argument)
        raise TypeError(message)

    def __copy__(self):
        r'''Copies numbered interval.

        Returns new numbered interval.
        '''
        return type(self)(self.number)

    def __eq__(self, argument):
        r'''Is true when `argument` is a numbered interval with number equal to that of
        this numbered interval. Otherwise false.

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            if self.number == argument.number:
                return True
        return False

    def __float__(self):
        r'''Changes numbered interval to float.

        Returns float.
        '''
        return float(self._number)

    def __hash__(self):
        r'''Hashes numbered interval.

        Returns integer.
        '''
        return hash(repr(self))

    def __int__(self):
        r'''Changes numbered interval to integer.

        Returns integer.
        '''
        return int(self._number)

    def __lt__(self, argument):
        r'''Is true when `argument` is a numbered interval with same direction number
        as this numbered interval and with number greater than that of this
        numbered interval. Otherwise false.

        Returns true or false.
        '''
        if not isinstance(argument, type(self)):
            message = 'must be numbered interval: {!r}.'
            message = message.format(argument)
            raise TypeError(message)
        if not self.direction_number == argument.direction_number:
            message = 'can only compare intervals of same direction.'
            raise ValueError(message)
        return abs(self.number) < abs(argument.number)

    def __neg__(self):
        r'''Negates numbered interval.

        Returns new numbered interval.
        '''
        return type(self)(-self._number)

    def __str__(self):
        r'''String representation of numbered interval.

        Returns string.
        '''
        return self._format_string

    def __sub__(self, argument):
        r'''Subtracts `argument` from numbered interval.

        Returns new numbered interval.
        '''
        if isinstance(argument, type(self)):
            number = self.number - argument.number
            return type(self)(number)
        message = 'must be {}: {!r}.'
        message = message.format(type(self), argument)
        raise TypeError(message)

    ### PRIVATE PROPERTIES ###

    @property
    def _format_string(self):
        return '{}{}'.format(self._direction_symbol, abs(self.number))

    ### PUBLIC PROPERTIES ###

    @property
    def direction_number(self):
        r'''Direction sign of numbered interval.

        ::

            >>> NumberedInterval(-14).direction_number
            -1

        Returns integer.
        '''
        return mathtools.sign(self.number)

    @property
    def direction_string(self):
        r'''Direction string of named interval.

        ::

            >>> NumberedInterval(-14).direction_string
            'descending'

        Returns ``'ascending'``, ``'descending'`` or none.
        '''
        if self.direction_number == -1:
            return 'descending'
        elif self.direction_number == 0:
            return None
        elif self.direction_number == 1:
            return 'ascending'

    @property
    def number(self):
        r'''Number of numbered interval.

        Returns number.
        '''
        return self._number

    @property
    def numbered_interval_number(self):
        r'''Number of numbered interval.

        ::

            >>> NumberedInterval(-14).numbered_interval_number
            -14

        Returns integer or float.
        '''
        return self._number

    @property
    def semitones(self):
        r'''Semitones corresponding to numbered interval.

        Returns nonnegative number.
        '''
        return self.number

    ### PUBLIC METHODS ###

    @classmethod
    def from_pitch_carriers(class_, pitch_carrier_1, pitch_carrier_2):
        '''Makes numbered interval from `pitch_carrier_1` and
        `pitch_carrier_2`.

        ::

            >>> NumberedInterval.from_pitch_carriers(
            ...     NamedPitch(-2),
            ...     NamedPitch(12),
            ...     )
            NumberedInterval(14)

        Returns numbered interval.
        '''
        from abjad.tools import pitchtools
        # get pitches
        pitch_1 = pitchtools.NamedPitch.from_pitch_carrier(pitch_carrier_1)
        pitch_2 = pitchtools.NamedPitch.from_pitch_carrier(pitch_carrier_2)
        # get difference in semitones
        number = pitchtools.NumberedPitch(pitch_2).pitch_number - \
            pitchtools.NumberedPitch(pitch_1).pitch_number
        # change 1.0, 2.0, ... into 1, 2, ...
        number = mathtools.integer_equivalent_number_to_integer(number)
        # return numbered interval
        return class_(number)

    def to_named_interval(self, staff_positions):
        r'''Changes numbered interval to named interval that encompasses
        `staff_positions`.

        ..  container:: example

            ::

                >>> numbered_interval = NumberedInterval(1)
                >>> numbered_interval.to_named_interval(2)
                NamedInterval('+m2')

        Returns named interval.
        '''
        from abjad.tools import pitchtools
        direction_number = mathtools.sign(self.number)
        if staff_positions == 1:
            quality_string = None
            if self.number % 12 == 11:
                quality_string = 'augmented'
            elif self.number % 12 == 0:
                quality_string = 'perfect'
            elif self.number % 12 == 1:
                quality_string = 'augmented'
            if not direction_number == 0:
                staff_positions *= direction_number
            if quality_string is None:
                # TODO: handle double-augmented named intervals
                return pitchtools.NamedInterval(self.number)
            named_interval = pitchtools.NamedInterval(
                quality_string, staff_positions)
            return named_interval
        named_interval_class_number = staff_positions % 7
        numbered_interval_class_number = abs(self.number) % 12
        if named_interval_class_number == 0:
            if numbered_interval_class_number == 9:
                quality_string = 'diminished'
            elif numbered_interval_class_number == 10:
                quality_string = 'minor'
            elif numbered_interval_class_number == 11:
                quality_string = 'major'
            elif numbered_interval_class_number == 0:
                quality_string = 'augmented'
        elif named_interval_class_number == 1:
            if numbered_interval_class_number == 11:
                quality_string = 'diminished'
            elif numbered_interval_class_number == 0:
                quality_string = 'perfect'
            elif numbered_interval_class_number == 1:
                quality_string = 'augmented'
        elif named_interval_class_number == 2:
            if numbered_interval_class_number == 0:
                quality_string = 'diminished'
            elif numbered_interval_class_number == 1:
                quality_string = 'minor'
            elif numbered_interval_class_number == 2:
                quality_string = 'major'
            elif numbered_interval_class_number == 3:
                quality_string = 'augmented'
        elif named_interval_class_number == 3:
            if numbered_interval_class_number == 2:
                quality_string = 'diminished'
            elif numbered_interval_class_number == 3:
                quality_string = 'minor'
            elif numbered_interval_class_number == 4:
                quality_string = 'major'
            elif numbered_interval_class_number == 5:
                quality_string = 'augmented'
        elif named_interval_class_number == 4:
            if numbered_interval_class_number == 4:
                quality_string = 'diminished'
            elif numbered_interval_class_number == 5:
                quality_string = 'perfect'
            elif numbered_interval_class_number == 6:
                quality_string = 'augmented'
        elif named_interval_class_number == 5:
            if numbered_interval_class_number == 6:
                quality_string = 'diminished'
            elif numbered_interval_class_number == 7:
                quality_string = 'perfect'
            elif numbered_interval_class_number == 8:
                quality_string = 'augmented'
        elif named_interval_class_number == 6:
            if numbered_interval_class_number == 7:
                quality_string = 'diminished'
            elif numbered_interval_class_number == 8:
                quality_string = 'minor'
            elif numbered_interval_class_number == 9:
                quality_string = 'major'
            elif numbered_interval_class_number == 10:
                quality_string = 'augmented'
        elif named_interval_class_number == 7:
            if numbered_interval_class_number == 9:
                quality_string = 'diminished'
            elif numbered_interval_class_number == 10:
                quality_string = 'minor'
            elif numbered_interval_class_number == 11:
                quality_string = 'major'
            elif numbered_interval_class_number == 0:
                quality_string = 'augmented'
        elif named_interval_class_number == 8:
            if numbered_interval_class_number == 11:
                quality_string = 'diminished'
            elif numbered_interval_class_number == 0:
                quality_string = 'perfect'
            elif numbered_interval_class_number == 1:
                quality_string = 'augmented'
        if not direction_number == 0:
            staff_positions *= direction_number
        named_interval = pitchtools.NamedInterval(
            quality_string,
            staff_positions,
            )
        return named_interval

    def transpose(self, pitch_carrier):
        r'''Transposes `pitch_carrier`.

        ..  container:: example

            Transposes chord:

            ::

                >>> chord = Chord("<c' e' g'>4")

            ::

                >>> interval = NumberedInterval(1)
                >>> interval.transpose(chord)
                Chord("<cs' f' af'>4")

        Returns new (copied) object of `pitch_carrier` type.
        '''
        from abjad.tools import pitchtools
        from abjad.tools import scoretools
        if isinstance(pitch_carrier, pitchtools.Pitch):
            number = pitch_carrier.pitch_number + self.semitones
            return type(pitch_carrier)(number)
        elif isinstance(pitch_carrier, numbers.Number):
            pitch_carrier = pitchtools.NumberedPitch(pitch_carrier)
            result = self.transpose(pitch_carrier)
            return result.pitch_number
        elif isinstance(pitch_carrier, scoretools.Note):
            new_note = copy.copy(pitch_carrier)
            number = pitchtools.NumberedPitch(pitch_carrier.written_pitch).pitch_number
            number += self.number
            new_pitch = pitchtools.NamedPitch(number)
            new_note.written_pitch = new_pitch
            return new_note
        elif isinstance(pitch_carrier, scoretools.Chord):
            new_chord = copy.copy(pitch_carrier)
            pairs = zip(new_chord.note_heads, pitch_carrier.note_heads)
            for new_nh, old_nh in pairs:
                number = \
                    pitchtools.NumberedPitch(old_nh.written_pitch).pitch_number
                number += self.number
                new_pitch = pitchtools.NamedPitch(number)
                new_nh.written_pitch = new_pitch
            return new_chord
        else:
            return pitch_carrier



# -*- coding: utf-8 -*-
import collections
import numbers
from abjad import Fraction
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import sequencetools


def make_notes(
    pitches,
    durations,
    decrease_durations_monotonically=True,
    use_messiaen_style_ties=False,
    ):
    r'''Makes notes according to `pitches` and `durations`.

    ..  container:: example

        **Example 1.** Cycles through `pitches` when the length of `pitches` is
        less than the length of `durations`:

        ::

            >>> notes = scoretools.make_notes([0], [(1, 16), (1, 8), (1, 8)])
            >>> notes
            Selection([Note("c'16"), Note("c'8"), Note("c'8")])

        ::

            >>> staff = Staff(notes)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'16
                c'8
                c'8
            }

    ..  container:: example

        **Example 2.** Cycles through `durations` when the length of `durations`
        is less than the length of `pitches`:

        ::

            >>> notes = scoretools.make_notes(
            ...     [0, 2, 4, 5, 7],
            ...     [(1, 16), (1, 8), (1, 8)],
            ...     )
            >>> notes
            Selection([Note("c'16"), Note("d'8"), Note("e'8"), Note("f'16"), Note("g'8")])

        ::

            >>> staff = Staff(notes)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'16
                d'8
                e'8
                f'16
                g'8
            }

    ..  container:: example

        **Example 3.** Creates ad hoc tuplets for nonassignable durations:

        ::

            >>> notes = scoretools.make_notes([0], [(1, 16), (1, 12), (1, 8)])
            >>> notes
            Selection([Note("c'16"), Tuplet(Multiplier(2, 3), "c'8"), Note("c'8")])

        ::

            >>> staff = Staff(notes)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'16
                \tweak edge-height #'(0.7 . 0)
                \times 2/3 {
                    c'8
                }
                c'8
            }

    ..  container:: example

        **Example 4.** Set ``decrease_durations_monotonically=True`` to express
        tied values in decreasing duration:

        ::

            >>> notes = scoretools.make_notes(
            ...     [0],
            ...     [(13, 16)],
            ...     decrease_durations_monotonically=True,
            ...     )
            >>> notes
            Selection([Note("c'2."), Note("c'16")])

        ::

            >>> staff = Staff(notes)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'2. ~
                c'16
            }

    ..  container:: example

        **Example 5.** Set ``decrease_durations_monotonically=False`` to
        express tied values in increasing duration:

        ::

            >>> notes = scoretools.make_notes(
            ...     [0],
            ...     [(13, 16)],
            ...     decrease_durations_monotonically=False,
            ...     )
            >>> notes
            Selection([Note("c'16"), Note("c'2.")])

        ::

            >>> staff = Staff(notes)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'16 ~
                c'2.
            }

    ..  container:: example

        **Example 6.** Uses Messiaen-style ties:

        ::

            >>> notes = scoretools.make_notes(
            ...     [0],
            ...     [(13, 16)],
            ...     use_messiaen_style_ties=True,
            ...     )
            >>> staff = Staff(notes)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'2.
                c'16 \repeatTie
            }

    ..  container:: example

        **Example 7.** Works with pitch segments:

        ::

            >>> segment = PitchSegment([-2, -1.5, 6, 7, -1.5, 7])
            >>> notes = scoretools.make_notes(segment, [(1, 8)])
            >>> staff = Staff(notes)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                bf8
                bqf8
                fs'8
                g'8
                bqf8
                g'8
            }

    Set `pitches` to a single pitch or a sequence of pitches.

    Set `durations` to a single duration or a list of durations.

    Returns selection.
    '''
    from abjad.tools import scoretools
    from abjad.tools import selectiontools
    if isinstance(pitches, str):
        pitches = pitches.split()
    if not isinstance(pitches, collections.Iterable):
        pitches = [pitches]
    if isinstance(durations, (numbers.Number, tuple)):
        durations = [durations]
    nonreduced_fractions = [mathtools.NonreducedFraction(_) for _ in durations]
    size = max(len(nonreduced_fractions), len(pitches))
    nonreduced_fractions = sequencetools.Sequence(nonreduced_fractions)
    nonreduced_fractions = nonreduced_fractions.repeat_to_length(size)
    pitches = sequencetools.Sequence(pitches).repeat_to_length(size)
    Duration = durationtools.Duration
    durations = Duration._group_nonreduced_fractions_by_implied_prolation(
        nonreduced_fractions)

    def _make_unprolated_notes(
        pitches,
        durations,
        decrease_durations_monotonically=decrease_durations_monotonically,
        use_messiaen_style_ties=False,
        ):
        assert len(pitches) == len(durations)
        result = []
        for pitch, duration in zip(pitches, durations):
            result.extend(
                scoretools.make_tied_leaf(
                    scoretools.Note,
                    duration,
                    pitches=pitch,
                    decrease_durations_monotonically=decrease_durations_monotonically,
                    use_messiaen_style_ties=use_messiaen_style_ties,
                    )
                )
        return result

    result = []
    for duration in durations:
        # get factors in denominator of duration group duration not 1 or 2
        factors = set(mathtools.factors(duration[0].denominator))
        factors.discard(1)
        factors.discard(2)
        ps = pitches[0:len(duration)]
        pitches = pitches[len(duration):]
        if len(factors) == 0:
            result.extend(
                _make_unprolated_notes(
                    ps,
                    duration,
                    decrease_durations_monotonically=decrease_durations_monotonically,
                    use_messiaen_style_ties=use_messiaen_style_ties,
                    )
                )
        else:
            # compute prolation
            denominator = duration[0].denominator
            numerator = mathtools.greatest_power_of_two_less_equal(denominator)
            multiplier = (numerator, denominator)
            ratio = 1 / Fraction(*multiplier)
            duration = [ratio * durationtools.Duration(d) for d in duration]
            ns = _make_unprolated_notes(
                ps,
                duration,
                decrease_durations_monotonically=decrease_durations_monotonically,
                use_messiaen_style_ties=use_messiaen_style_ties,
                )
            t = scoretools.Tuplet(multiplier, ns)
            result.append(t)

    # return result
    result = selectiontools.Selection(result)
    return result



# -*- coding: utf-8 -*-
from abjad.tools.quantizationtools.JobHandler import JobHandler


class SerialJobHandler(JobHandler):
    r'''Processes ``QuantizationJob`` instances sequentially.
    '''

    ### SPECIAL METHODS ###

    def __call__(self, jobs):
        r'''Calls serial job handler.

        Returns `jobs`.
        '''
        for job in jobs:
            job()
        return jobs



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_LogicalTie__leaves_grouped_by_immediate_parents_01():

    staff = Staff(2 * Measure((2, 8), "c'8 c'8"))
    leaves = select(staff).by_leaf()
    tie = spannertools.Tie()
    attach(tie, leaves)

    r'''
    \new Staff {
            \time 2/8
            c'8 ~
            c'8 ~
            \time 2/8
            c'8 ~
            c'8
    }
    '''

    logical_tie = inspect_(leaves[0]).get_logical_tie()
    groups = logical_tie._leaves_grouped_by_immediate_parents

    assert len(groups) == 2
    assert groups[0] == leaves[:2]
    assert groups[1] == leaves[2:]


def test_selectiontools_LogicalTie__leaves_grouped_by_immediate_parents_02():

    staff = Staff("c'8 ~ c'8 ~ c'8 ~ c'8")
    logical_tie = inspect_(staff[0]).get_logical_tie()
    groups = logical_tie._leaves_grouped_by_immediate_parents

    assert len(groups) == 1
    assert groups[0] == list(staff[:])


def test_selectiontools_LogicalTie__leaves_grouped_by_immediate_parents_03():

    note = Note("c'4")
    logical_tie = inspect_(note).get_logical_tie()
    groups = logical_tie._leaves_grouped_by_immediate_parents

    assert len(groups) == 1
    assert groups[0] == [note]



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_QEventProxy___init___01():
    q_event = quantizationtools.PitchedQEvent(130, [0])
    proxy = quantizationtools.QEventProxy(q_event, 0.5)
    assert proxy.q_event == q_event
    assert proxy.offset == durationtools.Offset(1, 2)


def test_quantizationtools_QEventProxy___init___02():
    q_event = quantizationtools.PitchedQEvent(130, [0, 1, 4])
    proxy = quantizationtools.QEventProxy(q_event, 100, 1000)
    assert proxy.q_event == q_event
    assert proxy.offset == durationtools.Offset(1, 30)



# -*- coding: utf-8 -*-
try:
    import funcsigs
except ImportError:
    import inspect as funcsigs
import inspect
import numbers
from abjad.tools.abctools import AbjadObject


class Expression(AbjadObject):
    r'''Expression.

    ..  container:: example expression

        Makes identity expression:

        ::

            >>> expression = Expression()

        ::

            >>> expression() is None
            True

        ::

            >>> expression(99)
            99

        ::

            >>> expression([99, 99.25])
            [99, 99.25]

    ..  container:: example expression

        Makes integer initialization expression:

        ::

            >>> expression = Expression(evaluation_template='int({})')

        ::

            >>> expression()
            0

        ::

            >>> expression(99)
            99

        ::

            >>> expression(99.25)
            99

    ..  container:: example expression

        Makes binary integer initialization expression:

        ::

            >>> expression = Expression(evaluation_template='int({}, base=2)')

        ::

            >>> expression('1')
            1

        ::

            >>> expression('11')
            3

        ::

            >>> expression('111')
            7

    ..  container:: example expression

        Makes three-integer addition expression:

        ::

            >>> expression = Expression(
            ...     argument_count=3,
            ...     evaluation_template='{} + {} + {}',
            ...     )

        ::

            >>> expression(1, 2, 3)
            6

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_argument_count',
        '_callbacks',
        '_evaluation_template',
        '_force_return',
        '_has_parentheses',
        '_is_composite',
        '_is_initializer',
        '_is_postfix',
        '_keywords',
        '_map_operand',
        '_markup_expression',
        '_module_names',
        '_name',
        '_next_name',
        '_precedence',
        '_proxy_class',
        '_string_template',
        '_subclass_hook',
        '_subexpressions',
        )

    _private_attributes_to_copy = []

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(
        self,
        argument_count=None,
        callbacks=None,
        evaluation_template=None,
        force_return=None,
        has_parentheses=None,
        is_composite=None,
        is_initializer=None,
        is_postfix=None,
        keywords=None,
        map_operand=None,
        markup_expression=None,
        module_names=None,
        name=None,
        next_name=None,
        precedence=None,
        proxy_class=None,
        string_template=None,
        subclass_hook=None,
        subexpressions=None,
        ):
        if argument_count is not None:
            assert isinstance(argument_count, int) and 0 <= argument_count
        self._argument_count = argument_count
        if callbacks is not None:
            callbacks = tuple(callbacks)
        self._callbacks = callbacks
        if not isinstance(evaluation_template, (str, type(None))):
            message = 'must be string or none: {!r}.'
            message = message.format(evaluation_template)
            raise TypeError(message)
        self._evaluation_template = evaluation_template
        self._force_return = force_return
        if has_parentheses is not None:
            has_parentheses = bool(has_parentheses)
        self._has_parentheses = has_parentheses
        if is_composite is not None:
            is_composite = bool(is_composite)
        self._is_composite = is_composite
        self._is_initializer = is_initializer
        if not isinstance(keywords, (dict, type(None))):
            message = 'keywords must be dictionary or none: {!r}.'
            message = message.format(keywords)
            raise TypeError(message)
        if is_postfix is not None:
            is_postfix = bool(is_postfix)
        self._is_postfix = is_postfix
        self._keywords = keywords
        if not isinstance(map_operand, (Expression, list, type(None))):
            message = 'must be expression, expression list or none: {!r}.'
            message = message.format(map_operand)
            raise TypeError(message)
        self._map_operand = map_operand
        if not isinstance(markup_expression, (Expression, type(None))):
            message = 'must be expression or none: {!r}.'
            message = message.format(markup_expression)
            raise TypeError(message)
        self._markup_expression = markup_expression
        self._module_names = module_names
        if not isinstance(name, (str, type(None))):
            message = 'name must be string or none: {!r}.'
            message = message.format(name)
            raise TypeError(name)
        self._name = name
        if not isinstance(next_name, (str, type(None))):
            message = 'next name must be string or none: {!r}.'
            message = message.format(next_name)
            raise TypeError(next_name)
        self._next_name = next_name
        self._precedence = precedence
        self._proxy_class = proxy_class
        if not isinstance(string_template, (str, type(None))):
            message = 'must be string or none: {!r}.'
            message = message.format(string_template)
            raise TypeError(message)
        self._string_template = string_template
        if not isinstance(subclass_hook, (str, type(None))):
            message = 'must be string or none: {!r}.'
            message = message.format(subclass_hook)
            raise TypeError(message)
        self._subclass_hook = subclass_hook
        if subexpressions is not None:
            subexpressions = tuple(subexpressions)
        self._subexpressions = subexpressions

    ### SPECIAL METHODS ###

    def __add__(self, i):
        r'''Gets proxy method or adds expressions.

        ..  container:: example expression

            Adds expressions:

            ::

                >>> expression_1 = Expression(
                ...     argument_count=3,
                ...     evaluation_template='{} + {} + {}',
                ...     )
                >>> expression_2 = Expression(
                ...     argument_count=2,
                ...     evaluation_template='{} + {}',
                ...     )
                >>> expression = expression_1 + expression_2
                >>> f(expression)
                expressiontools.Expression(
                    argument_count=2,
                    evaluation_template='{}.__add__({})',
                    is_composite=True,
                    markup_expression=expressiontools.Expression(
                        callbacks=[
                            expressiontools.Expression(
                                argument_count=2,
                                evaluation_template='[{}, {}]',
                                ),
                            expressiontools.Expression(
                                evaluation_template='abjad.markuptools.MarkupList',
                                is_initializer=True,
                                ),
                            expressiontools.Expression(
                                evaluation_template="{}.insert(i=1, item=markuptools.Markup(\n    contents=['+'],\n    ))",
                                force_return=True,
                                ),
                            expressiontools.Expression(
                                evaluation_template='{}.line()',
                                ),
                            ],
                        proxy_class=markuptools.MarkupList,
                        ),
                    string_template='{} + {}',
                    subexpressions=(
                        expressiontools.Expression(
                            argument_count=3,
                            evaluation_template='{} + {} + {}',
                            ),
                        expressiontools.Expression(
                            argument_count=2,
                            evaluation_template='{} + {}',
                            ),
                        ),
                    )

            ::

                >>> expression(1, 2, 3, 4, 5)
                15

        '''
        import abjad
        if not isinstance(i, Expression):
            proxy_method = self.__getattr__('__add__')
            return proxy_method(i)
        evaluation_template = '{}.__add__({})'
        if True:
            expression = abjad.Expression()
            callback = expression.make_callback(
                argument_count=2,
                evaluation_template='[{}, {}]'
                )
            expression = expression.append_callback(callback)
            expression = expression.markup_list()
            expression = expression.insert(1, abjad.Markup('+'))
            expression = expression.line()
        template = '{} + {}'
        lhs_module_names = self.module_names or []
        rhs_module_names = i.module_names or []
        module_names = lhs_module_names + rhs_module_names
        module_names = list(set(module_names))
        module_names.sort()
        module_names = module_names or None
        return type(self)(
            argument_count=2,
            evaluation_template=evaluation_template,
            is_composite=True,
            markup_expression=expression,
            module_names=module_names,
            proxy_class=self.proxy_class,
            string_template=template,
            subexpressions=[self, i],
            )

    def __call__(self, *arguments, **keywords):
        r'''Calls expression on `arguments` with `keywords`.

        ..  container:: example expression

            Calls identity expression:

            ::

                >>> expression = Expression()

            ::

                >>> expression() is None
                True

        ..  container:: example expression

            Calls markup expression:

            ::

                >>> expression = Expression()
                >>> expression = expression.markup()
                >>> expression = expression.bold()

            ::

                >>> expression('Allegro assai')
                Markup(contents=[MarkupCommand('bold', 'Allegro assai')])

        Returns ouput of last callback.
        '''
        arguments = list(arguments)
        results = []
        for subexpression in self.subexpressions or []:
            argument_count = subexpression.argument_count or 1
            arguments_ = []
            for i in range(argument_count):
                argument = arguments.pop(0)
                arguments_.append(argument)
            result = subexpression(*arguments_, **keywords)
            keywords = {}
            results.append(result)
        arguments = results or arguments
        if self.argument_count is not None:
            if len(arguments) != self.argument_count:
                message = '{} arguments (not {}) required: {!r}.'
                message = message.format(
                    self.argument_count,
                    len(arguments),
                    arguments,
                    )
                raise Exception(message)
        thread_arguments = False
        if self.evaluation_template is not None:
            result = self._evaluate(*arguments, **keywords)
            keywords = {}
        elif not arguments:
            result = None
        elif len(arguments) == 1:
            result = arguments[0]
        else:
            result = arguments
            thread_arguments = True
        for expression in self.callbacks or []:
            if expression.evaluation_template:
                if thread_arguments:
                    result = expression._evaluate(*result, **keywords)
                    thread_arguments = False
                else:
                    result = expression._evaluate(result, **keywords)
            else:
                if thread_arguments:
                    result = expression(*result, **keywords)
                    thread_arguments = False
                else:
                    result = expression(result, **keywords)
        return result

    def __dict__(self):
        r'''Gets attributes.

        Trivial implementation defined equal to ``dir(self)`` to satisfy PyPy.

        Returns list or strings.
        '''
        return dir(self)

    def __eq__(self, argument):
        r'''Is true when expression storage format equals `argument` storage
        format. Otherwise false.

        ..  container:: example

            ::

                >>> expression_1 = Expression().sequence()
                >>> expression_2 = Expression().sequence()
                >>> expression_1 == expression_2
                True

            ::

                >>> expression_1 = Expression().sequence()
                >>> expression_2 = new(expression_1)
                >>> expression_1 == expression_2
                True

        ..  container:: example

            ::

                >>> expression_1 = Expression().sequence()
                >>> expression_2 = Expression().sequence().reverse()
                >>> expression_1 == expression_2
                False

            ::

                >>> expression_1 = Expression().sequence()
                >>> expression_2 = Expression().markup()
                >>> expression_1 == expression_2
                False
                
            ::

                >>> expression_1 = Expression().sequence()
                >>> expression_1 == 'text'
                False

        Returns true or false.
        '''
        if not isinstance(argument, type(self)):
            return False
        expression_storage_format = format(self, 'storage')
        argument_storage_format = format(argument, 'storage')
        return expression_storage_format == argument_storage_format

    def __format__(self, format_specification=''):
        r'''Formats expression.

        ..  container:: example expression

            Formats identity expression:

            ::

                >>> expression = Expression()

            ::

                >>> f(expression)
                expressiontools.Expression()

        ..  container:: example expression

            Formats markup expression:

            ::

                >>> expression = Expression()
                >>> expression = expression.markup()
                >>> expression = expression.bold()

            ::

                >>> f(expression)
                expressiontools.Expression(
                    callbacks=[
                        expressiontools.Expression(
                            evaluation_template='abjad.markuptools.Markup',
                            is_initializer=True,
                            ),
                        expressiontools.Expression(
                            evaluation_template='{}.bold()',
                            ),
                        ],
                    proxy_class=markuptools.Markup,
                    )

        Returns string.
        '''
        superclass = super(Expression, self)
        return superclass.__format__(
            format_specification=format_specification,
            )

    def __getattr__(self, name):
        r'''Gets attribute `name`.

        Returns proxy method when proxy class is set.

        Returns normally when proxy class is not set.
        '''
        if self.__getattribute__('_proxy_class') is not None:
            if hasattr(self._proxy_class, name):
                proxy_object = self._proxy_class()
                if not hasattr(proxy_object, name):
                    message = 'proxy object {!r} has no attribute {!r}.'
                    message = message.format(proxy_object, name)
                    raise Exception(message)
                if not hasattr(proxy_object, '_expression'):
                    class_name = proxy_object.__name__
                    message = 'does not implement expression protocol: {}.'
                    message = message.format(class_name)
                    raise Exception(message)
                proxy_object._expression = self
                callable_ = getattr(proxy_object, name)
                assert callable(callable_), repr(callable_)
                if inspect.isfunction(callable_):
                    callable_.__dict__['frozen_expression'] = self
                return callable_
        message = '{} object has no attribute {!r}.'
        message = message.format(type(self).__name__, name)
        raise AttributeError(message)

    def __getitem__(self, argument):
        r'''Gets proxy method.
        '''
        proxy_method = self.__getattr__('__getitem__')
        return proxy_method(argument)

    def __hash__(self):
        r'''Hashes expression.

        Returns integer.
        '''
        superclass = super(Expression, self)
        return superclass.__hash__()

    def __iadd__(self, i):
        r'''Gets proxy method.
        '''
        proxy_method = self.__getattr__('__iadd__')
        return proxy_method(i)

    def __radd__(self, i):
        r'''Gets proxy method.
        '''
        proxy_method = self.__getattr__('__radd__')
        return proxy_method(i)

    def __repr__(self):
        r'''Gets interpreter representation.

        ..  container:: example expression

            Gets interpreter representation of identity expression:

            ::

                >>> expression = Expression()

            ::

                >>> expression
                Expression()

        ..  container:: example expression

            Gets interpreter representation of markup expression:

            ::

                >>> expression = Expression()
                >>> expression = expression.markup()
                >>> expression = expression.bold()

            ::

                >>> expression
                Expression(callbacks=[Expression(evaluation_template='abjad.markuptools.Markup', is_initializer=True), Expression(evaluation_template='{}.bold()')], proxy_class=Markup)

        Returns string.
        '''
        superclass = super(Expression, self)
        return superclass.__repr__()

    def __setitem__(self, i, argument):
        r'''Gets proxy method.
        '''
        proxy_method = self.__getattr__('__setitem__')
        return proxy_method(i, argument)

    def __str__(self):
        r'''Gets string representation of expression.

        ..  container:: example expression

            Gets string representation of identity expression:

            ::

                >>> expression = Expression()

            ::

                >>> str(expression)
                'Expression()'

        ..  container:: example expression

            Gets string representation of markup expression:

            ::

                >>> expression = Expression()
                >>> expression = expression.markup()
                >>> expression = expression.bold()

            ::

                >>> str(expression)
                "Expression(callbacks=[Expression(evaluation_template='abjad.markuptools.Markup', is_initializer=True), Expression(evaluation_template='{}.bold()')], proxy_class=Markup)"

        Returns string.
        '''
        superclass = super(Expression, self)
        return superclass.__str__()

    ### PRIVATE METHODS ###

    def _compile_callback_markup(
        self,
        name,
        direction=None,
        previous_callback=None,
        ):
        import abjad
        if previous_callback and previous_callback.next_name:
            name = previous_callback.next_name
        markup = name
        if isinstance(markup, str):
            markup = abjad.Markup(markup).bold()
        if not self.callbacks:
            return markup
        callback = self.callbacks[0]
        if callback.markup_expression is None:
            message = 'no markup expression found: {!r}.'
            message = message.format(callback)
            raise ValueError(message)
        if (previous_callback and
            previous_callback.is_composite and
            not callback.is_composite):
            markup = abjad.MarkupList(['(', markup, ')']).concat()
        markup = callback.markup_expression(markup)
        next_callback = None
        if 1 < len(self.callbacks):
            next_callback = self.callbacks[1]
        previous_precedence = callback.precedence or 0
        previous_callback = callback
        for callback in self.callbacks[1:]:
            if callback.markup_expression is None:
                message = 'callback {!r} has no markup expression.'
                message = message.format(callback)
                raise ValueError(message)
            if previous_callback and previous_callback.next_name:
                markup = previous_callback.next_name
                if isinstance(markup, str):
                    markup = abjad.Markup(markup).bold()
            current_precedence = callback.precedence or 0
            parenthesize_argument = False
            if (previous_precedence < current_precedence and
                not previous_callback.is_initializer):
                parenthesize_argument = True
            elif previous_callback.is_composite:
                parenthesize_argument = True
            if previous_callback and previous_callback.next_name:
                parenthesize_argument = False
            if parenthesize_argument:
                markup = abjad.MarkupList(['(', markup, ')']).concat()
            markup = callback.markup_expression(markup)
            previous_callback = callback
        markup = abjad.new(markup, direction=direction)
        return markup

    def _compile_callback_strings(self, name):
        string = name
        previous_callback = None
        for callback in self.callbacks or []:
            if previous_callback and previous_callback.next_name:
                string = previous_callback.next_name
            template = callback.string_template
            if template is None:
                message = 'callback {!r} has no string template.'
                message = message.format(callback)
                raise ValueError(message)
            try:
                string = template.format(string)
            except Exception as e:
                message = 'callback {!r} with template {!r}'
                message += ' and name {!r} raises {!r}.'
                message = message.format(callback, template, string, e)
                raise Exception(message)
            previous_callback = callback
        return string

    def _evaluate(self, *arguments, **keywords):
        import abjad
        assert self.evaluation_template
        if self.evaluation_template == 'map':
            return self._evaluate_map(*arguments)
        if self.subclass_hook:
            assert isinstance(self.subclass_hook, str)
            subclass_hook = getattr(self, self.subclass_hook)
            return subclass_hook(*arguments)
        argument_count = self.argument_count or 1
        if (len(arguments) != argument_count and
            len(arguments) != 0):
            message = '{} {} (not {}) required: {!r}.'
            message = message.format(
                argument_count,
                abjad.stringtools.pluralize('argument', argument_count),
                len(arguments),
                arguments,
                )
            raise Exception(message)
        globals_ = self._make_globals()
        statement = self.evaluation_template
        strings = []
        if self.is_initializer:
            for i, argument in enumerate(arguments):
                if argument is None:
                    continue
                string = '__argument_{i}'
                string = string.format(i=i)
                globals_[string] = argument
                strings.append(string)
            keywords_ = self.keywords or {}
            keywords_.update(keywords)
            for key, value in keywords_.items():
                value = self._to_evaluable_string(value)
                string = '{key}={value}'
                string = string.format(key=key, value=value)
                strings.append(string)
            strings = ', '.join(strings)
            statement = '{class_name}({strings})'
            class_name = self.evaluation_template
            statement = statement.format(
                class_name=class_name,
                strings=strings,
                )
        else:
            if not arguments:
                statement = statement.replace('{}', '')
            else:
                strings = []
                __argument_0 = arguments[0]
                for i, argument in enumerate(arguments):
                    string = '__argument_' + str(i)
                    globals_[string] = argument
                    strings.append(string)
                try:
                    statement = statement.format(*strings)
                except Exception as exception:
                    message = 'statement {!r} raises {!r}.'
                    message = message.format(statement, exception.args[0])
                    raise type(exception)(message)
        try:
            result = eval(statement, globals_)
        except Exception as exception:
            message = 'evaluable statement {!r} raises {!r}.'
            message = message.format(statement, exception.args[0])
            raise type(exception)(message)
        if self.force_return:
            result = __argument_0
        return result

    def _evaluate_map(self, *arguments):
        assert len(arguments) == 1, repr(arguments)
        assert self.map_operand is not None
        globals_ = self._make_globals()
        assert '__argument_0' not in globals_
        __argument_0 = arguments[0]
        class_ = type(__argument_0)
        map_operand = self.map_operand
        globals_['__argument_0'] = __argument_0
        globals_['class_'] = class_
        globals_['map_operand'] = map_operand
        statement = 'class_([map_operand(_) for _ in __argument_0])'
        try:
            result = eval(statement, globals_)
        except (NameError, SyntaxError, TypeError) as e:
            message = '{!r} raises {!r}.'
            message = message.format(statement, e)
            raise Exception(message)
        return result

    @classmethod
    def _frame_to_callback(
        class_,
        frame,
        evaluation_template=None,
        force_return=None,
        is_composite=None,
        is_initializer=None,
        is_postfix=None,
        keywords=None,
        map_operand=None,
        markup_expression=None,
        module_names=None,
        precedence=None,
        string_template=None,
        subclass_hook=None,
        ):
        if evaluation_template is None:
            evaluation_template = class_._get_evaluation_template(frame)
        result = class_._read_signature_decorator(frame)
        markup_expression = result['markup_expression'] or markup_expression
        string_template = result['string_template'] or string_template
        return class_(
            evaluation_template=evaluation_template,
            force_return=force_return,
            is_composite=is_composite,
            is_initializer=is_initializer,
            is_postfix=is_postfix,
            keywords=keywords,
            map_operand=map_operand,
            markup_expression=markup_expression,
            module_names=module_names,
            precedence=precedence,
            string_template=string_template,
            subclass_hook=subclass_hook,
            )
        
    @staticmethod
    def _get_callback(callback_name, function, function_self):
        callback = None
        callback_name = getattr(function, callback_name, None)
        if callback_name is not None:
            assert isinstance(callback_name, str), repr(callback_name)
            callback = getattr(function_self, callback_name, None)
            if callback is None:
                callback = getattr(Expression, callback_name, None)
            if callback is None:
                message = 'can not find callback {!r}.'
                message = message.format(callback_name)
                raise ValueError(message)
        return callback

    @staticmethod
    def _get_evaluation_template(frame, static_class=None):
        try:
            frame_info = inspect.getframeinfo(frame)
            function_name = frame_info.function
            arguments = Expression._wrap_arguments(
                frame,
                static_class=static_class,
                )
            template = '{{}}.{function_name}({arguments})'
            template = template.format(
                function_name=function_name,
                arguments=arguments,
                )
        finally:
            del frame
        return template

    @staticmethod
    def _get_method_name(
        function_name,
        function,
        function_self,
        argument_values,
        ):
        if getattr(function, 'method_name', None) is not None:
            return getattr(function, 'method_name')
        method_name_callback = Expression._get_callback(
            'method_name_callback', function, function_self,
            )
        if method_name_callback:
            return method_name_callback(**argument_values)
        return function_name

    @staticmethod
    def _make___add___markup_expression(argument):
        expression = Expression(is_postfix=True)
        expression = expression.markup()
        expression = expression.wrap_in_list()
        expression = expression.markup_list()
        expression = expression.append('+')
        expression = expression.append(str(argument))
        expression = expression.line()
        return expression

    @staticmethod
    def _make___add___string_template(argument):
        return '{} + ' + str(argument)

    @staticmethod
    def _make___getitem___markup_expression(argument):
        from abjad.tools import markuptools
        string = Expression._make_subscript_string(argument, markup=True)
        subscript_markup = markuptools.Markup(string).sub()
        expression = Expression(is_postfix=True)
        expression = expression.wrap_in_list()
        expression = expression.markup_list()
        expression = expression.append(subscript_markup)
        expression = expression.concat()
        return expression

    @staticmethod
    def _make___getitem___string_template(argument):
        string = Expression._make_subscript_string(argument, markup=False)
        return '{}' + string

    @staticmethod
    def _make___radd___markup_expression(argument):
        expression = Expression(is_postfix=True)
        expression = expression.markup()
        expression = expression.wrap_in_list()
        expression = expression.markup_list()
        expression = expression.insert(0, '+')
        expression = expression.insert(0, str(argument))
        expression = expression.line()
        return expression

    @staticmethod
    def _make___radd___string_template(argument):
        return str(argument) + ' + {}'

    def _make_evaluable_keywords(self, keywords):
        result = {}
        for key, value in keywords.items():
            if isinstance(value, type):
                value = self._to_evaluable_string(value)
            result[key] = value
        return result

    # TODO: eventually do not pass frame
    @staticmethod
    def _make_function_markup_expression(
        frame,
        function_name,
        argument_values,
        argument_list_callback,
        ):
        if argument_list_callback:
            arguments = argument_list_callback(**argument_values)
        else:
            arguments = Expression._wrap_arguments(frame)
        expression = Expression(has_parentheses=True)
        expression = expression.wrap_in_list()
        expression = expression.markup_list()
        expression = expression.insert(0, function_name + '(')
        if arguments:
            expression = expression.append(', ' + arguments + ')')
        else:
            expression = expression.append(')')
        expression = expression.concat()
        return expression

    # TODO: eventually do not pass frame
    @staticmethod
    def _make_function_string_template(
        frame,
        method_name,
        argument_values,
        argument_list_callback,
        ):
        if argument_list_callback:
            arguments = argument_list_callback(**argument_values)
        else:
            arguments = Expression._wrap_arguments(frame)
        if arguments:
            template = '{}({{}}, {})'
            template = template.format(method_name, arguments)
        else:
            template = method_name + '({})'
        return template

    def _make_globals(self):
        import abjad
        globals_ = {'abjad': abjad}
        globals_.update(abjad.__dict__.copy())
        module_names = self.module_names or ()
        for module_name in module_names:
            module = __import__(module_name)
            globals_[module_name] = module
        try:
            import experimental
            globals_.update(experimental.__dict__.copy())
        except ImportError:
            pass
        return globals_

    def _make_initializer_callback(
        self,
        class_,
        markup_expression=None,
        module_names=None,
        string_template=None,
        **keywords
        ):
        assert isinstance(class_, type), repr(class_)
        if not hasattr(class_, '_expression'):
            message = 'class does not implement expression protocol: {!r}.'
            message = message.format(class_)
            raise TypeError(message)
        parts = class_.__module__.split('.')
        if 'abjad' in parts:
            parts = [_ for _ in parts if _ != 'tools']
        evaluation_template = '.'.join(parts)
        keywords = self._make_evaluable_keywords(keywords)
        keywords = keywords or None
        return type(self)(
            evaluation_template=evaluation_template,
            is_initializer=True,
            keywords=keywords,
            markup_expression=markup_expression,
            module_names=module_names,
            string_template=string_template,
            )

    @staticmethod
    def _make_operator_markup_expression(
        method_name=None,
        subscript=None,
        superscript=None,
        ):
        from abjad.tools import markuptools
        expression = Expression()
        expression = expression.markup()
        expression = expression.wrap_in_list()
        expression = expression.markup_list()
        expression = expression.insert(0, method_name)
        if superscript is not None:
            superscript = markuptools.Markup(str(superscript))
            superscript = superscript.super()
            expression = expression.insert(1, superscript)
        if subscript is not None:
            subscript = markuptools.Markup(str(subscript))
            subscript = subscript.sub()
            expression = expression.insert(1, subscript)
        expression = expression.concat()
        return expression

    @staticmethod
    def _make_operator_string_template(
        method_name=None,
        subscript=None,
        superscript=None,
        ):
        template = method_name
        if superscript is not None:
            template += str(superscript)
        if subscript is not None:
            template += str(subscript)
        template += '({})'
        return template

    @staticmethod
    def _make_subscript_string(i, markup=False):
        if isinstance(i, int):
            if markup:
                subscript_string = '{i}'
            else:
                subscript_string = '[{i}]'
            start = stop = step = None
        elif isinstance(i, slice):
            if i.step is not None:
                raise NotImplementedError
            if i.start is None and i.stop is None:
                subscript_string = '[:]'
            elif i.start is None:
                subscript_string = '[:{stop}]'
            elif i.stop is None:
                subscript_string = '[{start}:]'
            else:
                subscript_string = '[{start}:{stop}]'
            start = i.start
            stop = i.stop
            step = i.step
        else:
            message = 'must be integer or slice: {!r}.'
            message = message.format(i)
            raise TypeError(message)
        subscript_string = subscript_string.format(
            i=i,
            start=start,
            stop=stop,
            step=step,
            )
        return subscript_string

    @staticmethod
    def _read_signature_decorator(frame):
        try:
            function_name = inspect.getframeinfo(frame).function
            values = inspect.getargvalues(frame)
            assert values.args[0] == 'self'
            function_self = values.locals['self']
            function = getattr(function_self, function_name)
            if not getattr(function, 'has_signature_decorator', False):
                return {'markup_expression': None, 'string_template': None}
            argument_names = values.args[1:]
            argument_values = {}
            for argument_name in argument_names:
                argument_value = values.locals[argument_name]
                argument_values[argument_name] = argument_value
            markup_expression_callback = Expression._get_callback(
                'markup_expression_callback', function, function_self)
            if markup_expression_callback is not None:
                string_template_callback = Expression._get_callback(
                    'string_template_callback', function, function_self)
                markup_expression = markup_expression_callback(
                    **argument_values)
                string_template = string_template_callback(**argument_values)
            elif getattr(function, 'is_operator', None):
                method_name = Expression._get_method_name(
                    function_name, function, function_self, argument_values)
                subscript = getattr(function, 'subscript', None)
                if subscript is not None:
                    subscript = values.locals[subscript]
                superscript = getattr(function, 'superscript', None)
                if superscript is not None:
                    superscript = values.locals[superscript]
                markup_expression = \
                    Expression._make_operator_markup_expression(
                    method_name=method_name,
                    subscript=subscript,
                    superscript=superscript)
                string_template = Expression._make_operator_string_template(
                    method_name=method_name,
                    subscript=subscript,
                    superscript=superscript,
                    )
            else:
                method_name = Expression._get_method_name(
                    function_name, function, function_self, argument_values)
                argument_list_callback = Expression._get_callback(
                    'argument_list_callback', function, function_self)
                # TODO: eventually do not pass frame
                markup_expression = \
                    Expression._make_function_markup_expression(
                    frame,
                    method_name,
                    argument_values,
                    argument_list_callback)
                # TODO: eventually do not pass frame
                string_template = Expression._make_function_string_template(
                    frame,
                    method_name,
                    argument_values,
                    argument_list_callback)
        finally:
            del frame
        return {
            'markup_expression': markup_expression,
            'string_template': string_template,
            }

    @staticmethod
    def _to_evaluable_string(argument):
        if argument is None:
            pass
        elif isinstance(argument, str):
            argument = repr(argument)
        elif isinstance(argument, numbers.Number):
            argument = str(argument)
        elif isinstance(argument, (list, tuple)):
            item_strings = []
            item_count = len(argument)
            for item in argument:
                item_string = Expression._to_evaluable_string(item)
                item_strings.append(item_string)
            items = ', '.join(item_strings)
            if isinstance(argument, list):
                argument = '[{}]'.format(items)
            elif isinstance(argument, tuple):
                if item_count == 1:
                    items += ','
                argument = '({})'.format(items)
            else:
                raise Exception(repr(argument))
        elif isinstance(argument, slice):
            argument = repr(argument)
        # abjad object
        elif not inspect.isclass(argument):
            try:
                argument = format(argument, 'storage')
            except (TypeError, ValueError):
                message = 'can not make storage format: {!r}.'
                message = message.format(argument)
                raise Exception(message)
        # abjad class
        elif inspect.isclass(argument) and 'abjad' in argument.__module__:
            argument = argument.__module__.split('.')
            argument = argument[-2:]
            argument = '.'.join(argument)
        # builtin class like tuple in classes=(tuple,)
        elif inspect.isclass(argument) and 'abjad' not in argument.__module__:
            argument = argument.__name__
        else:
            message = 'can not make evaluable string: {!r}.'
            message = message.format(argument)
            raise Exception(message)
        return argument

    @staticmethod
    def _wrap_arguments(frame, static_class=None):
        try:
            frame_info = inspect.getframeinfo(frame)
            function_name = frame_info.function
            values = inspect.getargvalues(frame)
            if static_class:
                method_name = frame.f_code.co_name
                static_method = getattr(static_class, method_name)
                #signature = inspect.signature(static_method)
                signature = funcsigs.signature(static_method)
                argument_names = values.args[:]
            else:
                assert values.args[0] == 'self'
                self = values.locals['self']
                function = getattr(self, function_name)
                #signature = inspect.signature(function)
                signature = funcsigs.signature(function)
                argument_names = values.args[1:]
            argument_strings = []
            for argument_name in argument_names:
                argument_value = values.locals[argument_name]
                parameter = signature.parameters[argument_name]
                if argument_value != parameter.default:
                    argument_string = '{argument_name}={argument_value}'
                    argument_value = Expression._to_evaluable_string(
                        argument_value)
                    argument_string = argument_string.format(
                        argument_name=argument_name,
                        argument_value=argument_value,
                        )
                    argument_strings.append(argument_string)
            arguments = ', '.join(argument_strings)
        finally:
            del frame
        return arguments

    ### PUBLIC PROPERTIES ###

    @property
    def argument_count(self):
        r'''Gets argument count.

        Defaults to none.

        Set to nonnegative integer or none.

        Returns nonnegative integer or none.
        '''
        return self._argument_count

    @property
    def callbacks(self):
        r'''Gets callbacks.

        ..  container:: example expression

            ::

                >>> expression = Expression()
                >>> expression = expression.markup()
                >>> expression = expression.bold()

            ::

                >>> for callback in expression.callbacks:
                ...     callback
                Expression(evaluation_template='abjad.markuptools.Markup', is_initializer=True)
                Expression(evaluation_template='{}.bold()')

        ..  container:: example expression

            Defaults to none:

            ::

                >>> expression = Expression()
                >>> expression.callbacks is None
                True

        Set to callbacks or none.

        ..  container:: example expression

            Returns list or none:

            ::

                >>> expression = Expression()
                >>> expression = expression.markup()
                >>> expression = expression.bold()

            ::

                >>> isinstance(expression.callbacks, list)
                True

        '''
        if self._callbacks:
            return list(self._callbacks)

    @property
    def evaluation_template(self):
        r'''Gets evaluation template.

        Defaults to none.

        Set to string.

        Returns string.
        '''
        return self._evaluation_template

    @property
    def force_return(self):
        r'''Is true when expression should return primary input argument.
        Otherwise false.

        Defaults to none.

        Set to true, false or none.

        Returns true, false or none.
        '''
        return self._force_return

    @property
    def has_parentheses(self):
        r'''Is true when expression has parentheses. Otherwise false.

        Defaults to none.

        Set to true, false or none.

        Returns true, false or none.
        '''
        return self._has_parentheses

    @property
    def is_composite(self):
        r'''Is true when expression is composite. Otherwise false.

        Defaults to none.

        Set to true, false or none.

        Returns true, false or none.
        '''
        return self._is_composite

    @property
    def is_initializer(self):
        r'''Is true when expression is initializer. Otherwise false.

        Defaults to none.

        Set to true, false or none.

        Returns true, false or none.
        '''
        return self._is_initializer

    @property
    def is_postfix(self):
        r'''Is true when expression is postfix. Otherwise false.

        Defaults to none.

        Set to true, false or none.

        Returns true, false or none.
        '''
        return self._is_postfix

    @property
    def keywords(self):
        r'''Gets keywords.

        Defaults to none.

        Set to dictionary or none.

        Returns dictionary or none.
        '''
        return self._keywords

    @property
    def map_operand(self):
        r'''Gets expression to map.

        Defaults to none.

        Set to expression or none.

        Returns expression or none.
        '''
        return self._map_operand

    @property
    def markup_expression(self):
        r'''Gets markup expression.

        Defaults to none.

        Set to markup or none.

        Returns markup or none.
        '''
        return self._markup_expression

    @property
    def module_names(self):
        r'''Gets module names.

        Defaults to none.

        Set to strings or none.

        Returns strings or none.
        '''
        return self._module_names

    @property
    def name(self):
        r'''Gets name.

        ..  container:: example expression

            Preserves name after initializer callback:

            ::

                >>> expression = Expression(name='J')
                >>> expression.name
                'J'

            ::

                >>> expression = expression.sequence()
                >>> expression.name
                'J'

        Defaults to none.

        Set to string or none.

        Returns string or none.
        '''
        return self._name

    @property
    def next_name(self):
        r'''Gets next name.

        Defaults to none.

        Set to string or none.

        Returns string or none.
        '''
        return self._next_name

    @property
    def precedence(self):
        r'''Gets precedence.

        Defaults to none.

        Set to integer or none.

        Returns integer or none.
        '''
        return self._precedence

    @property
    def proxy_class(self):
        r'''Gets proxy class.

        Defaults to none.

        Set to class or none.

        Returns class or none.
        '''
        return self._proxy_class

    @property
    def string_template(self):
        r'''Gets string template.

        Defaults to none.

        Set to string or none.

        Returns string or none.
        '''
        return self._string_template

    @property
    def subclass_hook(self):
        r'''Gets subclass hook.

        Only to be set by expression subclasses.

        Set to name of custom evaluation method.

        Defaults to none.

        Set to string or none.

        Returns string or none.
        '''
        return self._subclass_hook

    @property
    def subexpressions(self):
        r'''Gets subexpressions.

        Defaults to none.

        Set to expressions or none.

        Returns list of expressions or none.
        '''
        return self._subexpressions

    ### PUBLIC METHODS ###

    def append_callback(self, callback):
        r'''Appends callback to expression.

        ..  container:: example expression

            ::

                >>> expression = Expression()
                >>> expression.callbacks is None
                True

            ::

                
                >>> callback = Expression(evaluation_template='int({})')
                >>> expression = expression.append_callback(callback)
                >>> for callback in expression.callbacks:
                ...     callback
                Expression(evaluation_template='int({})')

            ::

                >>> callback = Expression(evaluation_template='{}**2')
                >>> expression = expression.append_callback(callback)
                >>> for expression in expression.callbacks:
                ...     expression
                Expression(evaluation_template='int({})')
                Expression(evaluation_template='{}**2')

        Returns new expression.
        '''
        from abjad.tools import topleveltools
        callbacks = self.callbacks or []
        callbacks = callbacks + [callback]
        return topleveltools.new(self, callbacks=callbacks)

    def establish_equivalence(self, name):
        r'''Makes new expression with `name`.

        ..  container:: example expression

            ::

                >>> expression = Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression.rotate(n=1)
                >>> expression = expression.rotate(n=2)
                >>> expression = expression.establish_equivalence(name='Q')

            ::

                >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([7, 10.5, 7, 10, 10.5, 6])

            ::

                >>> expression.get_string()
                'Q = r2(r1(J))'

            ::

                >>> markup = expression.get_markup()
                >>> show(markup) # doctest: +SKIP

            ..  doctest::

                >>> f(markup)
                \markup {
                    \line
                        {
                            \bold
                                Q
                            =
                            \concat
                                {
                                    r
                                    \sub
                                        2
                                    \concat
                                        {
                                            r
                                            \sub
                                                1
                                            \bold
                                                J
                                        }
                                }
                        }
                    }

        Returns new expression.
        '''
        import abjad
        expression = Expression()
        expression = expression.wrap_in_list()
        expression = expression.markup_list()
        expression = expression.insert(0, abjad.Markup(name).bold())
        expression = expression.insert(1, '=')
        expression = expression.line()
        template = '{name} = {{}}'
        template = template.format(name=name)
        callback = self.make_callback(
            evaluation_template='{}',
            is_composite=True,
            markup_expression=expression,
            next_name=name,
            string_template=template,
            )
        return self.append_callback(callback)

    def get_markup(self, direction=None, name=None):
        r'''Gets markup.

        Returns markup or none.
        '''
        import abjad
        argument_count = self.argument_count or 1
        markup = None
        if argument_count <= 1:
            if name is None:
                name = self.name
            if name is None:
                for callback in self.callbacks or []:
                    if callback.name is not None:
                        name = callback.name
                        break
            if name is None:
                message = 'expression name not found: {!r}.'
                message = message.format(self)
                raise ValueError(message)
            markup = self._compile_callback_markup(name)
        else:
            markups = []
            for subexpression in self.subexpressions or []:
                markup = subexpression.get_markup()
                markups.append(markup)
            if len(markups) != argument_count:
                message = 'argument count of {} with {} markups found.'
                message = message.format(argument_count, len(markups))
                raise ValueError(message)
            markup = self.markup_expression(*markups)
            markup = self._compile_callback_markup(
                markup,
                previous_callback=self,
                )
        if markup is not None:
            markup = abjad.new(markup, direction=direction)
        return markup

    def get_string(self, name=None):
        r'''Gets string.

        ..  container:: example

            Gets string for sequence expression:

            ..  container:: example expression

                Without name:

                ::

                    >>> expression = sequence()
                    >>> expression = expression.reverse()
                    >>> expression = expression.rotate(n=2)

                ::

                    >>> expression([1, 2, 3, 4, 5, 6])
                    Sequence([2, 1, 6, 5, 4, 3])

                ::

                    >>> Expression.get_string(expression, name='J')
                    'r2(R(J))'

            ..  container:: example expression

                With name:

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.sequence()
                    >>> expression = expression.reverse()
                    >>> expression = expression.rotate(n=2)

                ::

                    >>> expression([1, 2, 3, 4, 5, 6])
                    Sequence([2, 1, 6, 5, 4, 3])

                ::

                    >>> Expression.get_string(expression)
                    'r2(R(J))'

                Overrides name:

                ::

                    >>> Expression.get_string(expression, name='K')
                    'r2(R(K))'

        ..  container:: example

            Gets string for segment expression:

            ..  container:: example expression

                Without name:

                ::

                    >>> expression = Expression()
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.invert()
                    >>> expression = expression.rotate(n=2)

                ::

                    >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([1.5, 5, 2, 1.5, 6, 5])

                ::

                    >>> Expression.get_string(expression, name='J')
                    'r2(I(J))'

            ..  container:: example expression

                With name:

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.invert()
                    >>> expression = expression.rotate(n=2)

                ::

                    >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([1.5, 5, 2, 1.5, 6, 5])

                ::

                    >>> Expression.get_string(expression)
                    'r2(I(J))'

                Overrides name:

                ::

                    >>> Expression.get_string(expression, name='K')
                    'r2(I(K))'

        Returns string or none.
        '''
        argument_count = self.argument_count or 1
        if argument_count <= 1:
            if name is None:
                name = self.name
            if name is None:
                for callback in self.callbacks or []:
                    if callback.name is not None:
                        name = callback.name
                        break
            if name is None:
                message = 'expression name not found: {!r}.'
                message = message.format(self)
                raise ValueError(message)
            return self._compile_callback_strings(name)
        else:
            strings = []
            for subexpression in self.subexpressions or []:
                string = subexpression.get_string()
                strings.append(string)
            if len(strings) != argument_count:
                message = 'argument count of {} with {} strings found.'
                message = message.format(argument_count, len(strings))
                raise ValueError(message)
            template = self.string_template
            if template is None:
                message = 'expression has no string template: {!r}.'
                message = message.format(self)
                raise ValueError(message)
            try:
                string = template.format(*strings)
            except Exception as e:
                message = '{!r} with template {!r} and names {!r} raises {!r}.'
                message = message.format(self, template, names, e)
                raise Exception(message)
            return self._compile_callback_strings(string)

    def iterate(self, **keywords):
        r'''Makes iterate expression.

        ..  container:: example

            Makes expression to iterate leaves:

            ..  container:: example

                ::

                    >>> staff = Staff()
                    >>> staff.append(Measure((2, 8), "<c' bf'>8 <g' a'>8"))
                    >>> staff.append(Measure((2, 8), "af'8 r8"))
                    >>> staff.append(Measure((2, 8), "r8 gf'8"))
                    >>> show(staff) # doctest: +SKIP

                ..  doctest::

                    >>> f(staff)
                    \new Staff {
                        {
                            \time 2/8
                            <c' bf'>8
                            <g' a'>8
                        }
                        {
                            af'8
                            r8
                        }
                        {
                            r8
                            gf'8
                        }
                    }

            ..  container:: example expression

                ::

                    >>> expression = Expression()
                    >>> expression = expression.iterate()
                    >>> expression = expression.by_leaf()

                ::

                    >>> for leaf in expression(staff):
                    ...     leaf
                    ...
                    Chord("<c' bf'>8")
                    Chord("<g' a'>8")
                    Note("af'8")
                    Rest('r8')
                    Rest('r8')
                    Note("gf'8")

        Returns expression.
        '''
        import abjad
        class_ = abjad.agenttools.IterationAgent
        callback = self._make_initializer_callback(class_, **keywords)
        expression = self.append_callback(callback)
        return abjad.new(expression, proxy_class=class_)

    def label(self, **keywords):
        r'''Makes label expression.

        ..  container:: example

            Makes expression to label logical tie durations:

            ..  container:: example

                ::

                    >>> staff = Staff(r"c'4. d'8 ~ d'4. e'16 [ ef'16 ]")

            ..  container:: example expression

                ::

                    >>> expression = Expression()
                    >>> expression = expression.label()
                    >>> expression = expression.with_durations()

                ::

                    >>> expression(staff)
                    >>> show(staff) # doctest: +SKIP

                ..  doctest::

                    >>> f(staff)
                    \new Staff {
                        c'4.
                            ^ \markup {
                                \small
                                    3/8
                                }
                        d'8 ~
                            ^ \markup {
                                \small
                                    1/2
                                }
                        d'4.
                        e'16 [
                            ^ \markup {
                                \small
                                    1/16
                                }
                        ef'16 ]
                            ^ \markup {
                                \small
                                    1/16
                                }
                    }

        Returns expression.
        '''
        import abjad
        class_ = abjad.agenttools.LabelAgent
        callback = self._make_initializer_callback(class_, **keywords)
        expression = self.append_callback(callback)
        return abjad.new(expression, proxy_class=class_)

    @staticmethod
    def make_callback(
        argument_count=None,
        evaluation_template=None,
        force_return=None,
        has_parentheses=None,
        is_composite=None,
        is_initializer=None,
        is_postfix=None,
        keywords=None,
        map_operand=None,
        markup_expression=None,
        module_names=None,
        next_name=None,
        precedence=None,
        string_template=None,
        ):
        r'''Makes callback.

        Returns expression.
        '''
        return Expression(
            argument_count=argument_count,
            evaluation_template=evaluation_template,
            force_return=force_return,
            has_parentheses=has_parentheses,
            is_composite=is_composite,
            is_initializer=is_initializer,
            is_postfix=is_postfix,
            keywords=keywords,
            map_operand=map_operand,
            markup_expression=markup_expression,
            module_names=module_names,
            next_name=next_name,
            precedence=precedence,
            string_template=string_template,
            )

    def markup(self, **keywords):
        r'''Makes markup expression.

        ..  container:: example expression

            Makes expression to make bold markup:

            ::

                >>> expression = Expression()
                >>> expression = expression.markup()
                >>> expression = expression.bold()

            ::

                >>> markup = expression('Allegro assai')
                >>> show(markup) # doctest: +SKIP

            ..  doctest::

                >>> f(markup)
                \markup {
                    \bold
                        "Allegro assai"
                    }

        Returns expression.
        '''
        from abjad.tools import markuptools
        from abjad.tools import topleveltools
        class_ = markuptools.Markup
        callback = self._make_initializer_callback(class_, **keywords)
        expression = self.append_callback(callback)
        return topleveltools.new(expression, proxy_class=class_)

    def markup_list(self, **keywords):
        r'''Makes markup list expression.

        ..  container:: example expression

            Makes expression to concatenate markups in markup list:

            ::

                >>> expression = Expression()
                >>> expression = expression.markup_list()
                >>> expression = expression.concat(direction=Up)

            ::

                >>> downbow = Markup.musicglyph('scripts.downbow')
                >>> hspace = Markup.hspace(1)
                >>> upbow = Markup.musicglyph('scripts.upbow')
                >>> markups = [downbow, hspace, upbow]
                >>> markup = expression(markups)
                >>> show(markup) # doctest: +SKIP

            ..  doctest::

                >>> f(markup)
                ^ \markup {
                    \concat
                        {
                            \musicglyph
                                #"scripts.downbow"
                            \hspace
                                #1
                            \musicglyph
                                #"scripts.upbow"
                        }
                    }

        ..  container:: example expression

            ::

                >>> expression = Expression()
                >>> expression = expression.markup_list()

            Expression works with zero arguments:

            ::

                >>> expression()
                MarkupList(items=[])

            Expression works with one argument:

            ::

                >>> expression(['Allegro', 'ma non troppo'])
                MarkupList(items=[Markup(contents=['Allegro']), Markup(contents=['ma non troppo'])])

            Expression raises exception on more than one argument:

            ::

                >>> expression('Allegro', 'ma', 'non', 'troppo')
                Traceback (most recent call last):
                    ...
                Exception: 1 argument (not 4) required: ('Allegro', 'ma', 'non', 'troppo').

        Returns expression.
        '''
        from abjad.tools import markuptools
        from abjad.tools import topleveltools
        class_ = markuptools.MarkupList
        callback = self._make_initializer_callback(class_, **keywords)
        expression = self.append_callback(callback)
        return topleveltools.new(expression, proxy_class=class_)

    def pitch_class_segment(self, **keywords):
        r'''Makes pitch-class segment expression.

        ..  container:: example

            Makes expression to transpose pitch-class segment:

            ::

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> J = PitchClassSegment(items=items)
                >>> J
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            ::

                >>> show(J) # doctest: +SKIP

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.transpose(n=13)

                ::

                    >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([11, 11.5, 7, 8, 11.5, 8])

                ::

                    >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        b'8
                            ^ \markup {
                                \concat
                                    {
                                        T
                                        \sub
                                            13
                                        \bold
                                            J
                                    }
                                }
                        bqs'8
                        g'8
                        af'8
                        bqs'8
                        af'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        Returns expression.
        '''
        import abjad
        class_ = abjad.PitchClassSegment
        callback = self._make_initializer_callback(
            class_,
            markup_expression=type(self)().markup(),
            string_template='{}',
            **keywords
            )
        expression = self.append_callback(callback)
        return abjad.new(expression, proxy_class=class_)

    def sequence(self, **keywords):
        r'''Makes sequence expression.

        ..  container:: example expression

            Makes expression to initialize, flatten and reverse sequence:

            ::

                >>> expression = sequence()
                >>> expression = expression.reverse()
                >>> expression = expression.flatten()

            ::

                >>> expression([1, 2, 3, [4, 5, [6]]])
                Sequence([4, 5, 6, 3, 2, 1])

        Returns expression.
        '''
        import abjad
        class_ = abjad.Sequence
        callback = self._make_initializer_callback(
            class_,
            markup_expression=type(self)().markup(),
            string_template='{}',
            **keywords
            )
        expression = self.append_callback(callback)
        return abjad.new(expression, proxy_class=class_)

    def wrap_in_list(self):
        r'''Makes expression to wrap argument in list.

        ..  container:: example expression

            ::

                >>> expression = Expression()
                >>> expression = expression.wrap_in_list()

            ::

                >>> expression(Markup('Allegro assai'))
                [Markup(contents=['Allegro assai'])]

            ::

                >>> f(expression)
                expressiontools.Expression(
                    callbacks=[
                        expressiontools.Expression(
                            evaluation_template='[{}]',
                            ),
                        ],
                    )

        Returns expression.
        '''
        callback = self.make_callback(evaluation_template='[{}]')
        return self.append_callback(callback)



# -*- coding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_get_next_measure_from_component_01():

    staff = Staff()
    staff.append(Container("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"))
    staff.extend("abj: | 2/8 g'8 a'8 || 2/8 b'8 c''8 |")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                {
                    \time 2/8
                    c'8
                    d'8
                }
                {
                    e'8
                    f'8
                }
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )

    leaves = select(staff).by_leaf()
    assert scoretools.get_next_measure_from_component(staff) is staff[0][0]
    assert scoretools.get_next_measure_from_component(staff[0]) is staff[0][0]
    assert scoretools.get_next_measure_from_component(staff[0][0]) is staff[0][1]
    assert scoretools.get_next_measure_from_component(staff[0][1]) is staff[1]
    assert scoretools.get_next_measure_from_component(staff[1]) is staff[2]
    assert scoretools.get_next_measure_from_component(staff[2]) is None
    assert scoretools.get_next_measure_from_component(
        leaves[0]) is staff[0][0]
    assert scoretools.get_next_measure_from_component(
        leaves[1]) is staff[0][0]
    assert scoretools.get_next_measure_from_component(
        leaves[2]) is staff[0][1]
    assert scoretools.get_next_measure_from_component(
        leaves[3]) is staff[0][1]
    assert scoretools.get_next_measure_from_component(
        leaves[4]) is staff[1]
    assert scoretools.get_next_measure_from_component(
        leaves[5]) is staff[1]
    assert scoretools.get_next_measure_from_component(
        leaves[6]) is staff[2]
    assert scoretools.get_next_measure_from_component(
        leaves[7]) is staff[2]


def test_scoretools_get_next_measure_from_component_02():
    r'''Can retrieve first measure in a Python list.
    '''

    components = [Note("c'4"), Measure((2, 8), "c'8 d'8")]

    result = scoretools.get_next_measure_from_component(components)
    assert result is components[1]



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Staff___setitem___01():
    staff = Staff([Note("c'4"),
            Rest((1, 4)),
            Chord([2, 3, 4], (1, 4)),
            scoretools.Skip((1, 4)),
            scoretools.FixedDurationTuplet(Duration(5, 16), Note(0, (1, 16)) * 4)])
    assert len(staff) == 5
    assert inspect_(staff).is_well_formed()
    assert isinstance(staff[0], Note)
    assert isinstance(staff[1], Rest)
    assert isinstance(staff[2], Chord)
    assert isinstance(staff[3], scoretools.Skip)
    assert isinstance(staff[4], scoretools.FixedDurationTuplet)
    staff[1] = Chord([12, 13, 15], (1, 4))
    assert len(staff) == 5
    assert inspect_(staff).is_well_formed()
    assert isinstance(staff[0], Note)
    assert isinstance(staff[1], Chord)
    assert isinstance(staff[2], Chord)
    assert isinstance(staff[3], scoretools.Skip)
    assert isinstance(staff[4], scoretools.FixedDurationTuplet)
    staff[0] = Rest((1, 4))
    assert len(staff) == 5
    assert inspect_(staff).is_well_formed()
    assert isinstance(staff[0], Rest)
    assert isinstance(staff[1], Chord)
    assert isinstance(staff[2], Chord)
    assert isinstance(staff[3], scoretools.Skip)
    assert isinstance(staff[4], scoretools.FixedDurationTuplet)
    staff[-2] = scoretools.FixedDurationTuplet(Duration(2, 8), Note(0, (1, 8)) * 3)
    assert len(staff) == 5
    assert inspect_(staff).is_well_formed()
    assert isinstance(staff[0], Rest)
    assert isinstance(staff[1], Chord)
    assert isinstance(staff[2], Chord)
    assert isinstance(staff[3], scoretools.FixedDurationTuplet)
    assert isinstance(staff[4], scoretools.FixedDurationTuplet)
    staff[-1] = Note(13, (1, 4))
    assert len(staff) == 5
    assert inspect_(staff).is_well_formed()
    assert isinstance(staff[0], Rest)
    assert isinstance(staff[1], Chord)
    assert isinstance(staff[2], Chord)
    assert isinstance(staff[3], scoretools.FixedDurationTuplet)
    assert isinstance(staff[4], Note)
    staff[-3] = scoretools.Skip((1, 4))
    assert len(staff) == 5
    assert inspect_(staff).is_well_formed()
    assert isinstance(staff[0], Rest)
    assert isinstance(staff[1], Chord)
    assert isinstance(staff[2], scoretools.Skip)
    assert isinstance(staff[3], scoretools.FixedDurationTuplet)
    assert isinstance(staff[4], Note)


def test_scoretools_Staff___setitem___02():
    r'''Reassign the *entire* contents of staff.
    '''
    staff = Staff(Note("c'4") * 4)
    assert staff._contents_duration == Duration(4, 4)
    staff[:] = Note(0, (1, 8)) * 4
    assert staff._contents_duration == Duration(4, 8)


def test_scoretools_Staff___setitem___03():
    r'''Item-assign an empty container to staff.
    '''
    staff = Staff(Note("c'4") * 4)
    staff[0] = Voice([])


def test_scoretools_Staff___setitem___04():
    r'''Slice-assign empty containers to staff.
    '''
    staff = Staff(Note("c'4") * 4)
    staff[0:2] = [Voice([]), Voice([])]


def test_scoretools_Staff___setitem___05():
    r'''Bark when user assigns a slice to an item.
    '''

    staff = Staff(Note("c'4") * 4)

    statement = 'staff[0] = [Note(2, (1, 4)), Note(2, (1, 4))]'
    assert pytest.raises(AssertionError, statement)


def test_scoretools_Staff___setitem___06():
    r'''Bark when user assigns an item to a slice.
    '''

    staff = Staff(Note("c'4") * 4)

    assert pytest.raises(Exception, 'staff[0:2] = Note(2, (1, 4))')


def test_scoretools_Staff___setitem___07():
    r'''Slice-assign notes.
    '''

    staff = Staff(Note(0, (1, 8)) * 8)
    staff[0:4] = Note(2, (1, 8)) * 4
    assert len(staff) == 8
    for x in staff[0:4]:
        assert x.written_pitch.numbered_pitch._pitch_number == 2
    for x in staff[4:8]:
        assert x.written_pitch.numbered_pitch._pitch_number == 0
    assert inspect_(staff).is_well_formed()


def test_scoretools_Staff___setitem___08():
    r'''Slice-assign chords.
    '''

    staff = Staff(Note(0, (1, 8)) * 8)
    staff[0:4] = Chord([2, 3, 4], (1, 4)) * 4
    assert len(staff) == 8
    for x in staff[0:4]:
        assert x.written_duration == Duration(1, 4)
    for x in staff[4:8]:
        assert x.written_duration == Duration(1, 8)
    assert inspect_(staff).is_well_formed()


def test_scoretools_Staff___setitem___09():
    r'''Slice-assign tuplets.
    '''

    staff = Staff(Note(0, (1, 8)) * 8)
    staff[0:4] = scoretools.FixedDurationTuplet(
        Duration(2, 8), Note(0, (1, 8)) * 3) * 2
    assert len(staff) == 6
    for i, x in enumerate(staff):
        if i in [0, 1]:
            assert isinstance(x, scoretools.FixedDurationTuplet)
        else:
            assert isinstance(x, Note)
    assert inspect_(staff).is_well_formed()


def test_scoretools_Staff___setitem___10():
    r'''Slice-assign measures.
    '''

    staff = Staff(Note(0, (1, 8)) * 8)
    staff[0:4] = Measure((2, 8), Note(0, (1, 8)) * 2) * 2
    assert len(staff) == 6
    for i, x in enumerate(staff):
        if i in [0, 1]:
            assert isinstance(x, Measure)
        else:
            assert isinstance(x, Note)
    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadValueObject
from abjad.tools import mathtools
from abjad.tools import selectiontools
from abjad.tools import sequencetools


class GroupByPitchCallback(AbjadValueObject):
    r'''Group-by-pitch selector callback.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Callbacks'

    __slots__ = (
        '_allow_discontiguity',
        )

    ### INITIALIZER ###

    def __init__(self, allow_discontiguity=False):
        if allow_discontiguity is not None:
            allow_discontiguity = bool(allow_discontiguity)
        self._allow_discontiguity = allow_discontiguity

    ### SPECIAL METHODS ###

    def __call__(
        self,
        argument,
        rotation=None,
        ):
        r'''Calls selector callback on `argument`.

        Returns tuple of selections.
        '''
        import abjad
        assert isinstance(argument, tuple), repr(argument)
        if len(argument) == 1 and isinstance(argument[0], abjad.Selection):
            selection = argument[0]
        else:
            selection = abjad.select(argument)
        selections = selection.group_by(self._get_written_pitches)
        selections = self._map_contiguity(selections)
        selections = [selectiontools.Selection(_) for _ in selections]
        return tuple(selections)
    
    ### PRIVATE METHODS ###

    @staticmethod
    def _get_written_pitches(argument):
        import abjad
        if isinstance(argument, abjad.Note):
            return argument.written_pitch
        elif isinstance(argument, abjad.Chord):
            return argument.written_pitches
        elif (isinstance(argument, abjad.selectiontools.LogicalTie) and
            isinstance(argument.head, abjad.Note)):
            return argument.head.written_pitch
        elif (isinstance(argument, abjad.selectiontools.LogicalTie) and
            isinstance(argument.head, abjad.Chord)):
            return argument.head.written_pitches
        else:
            return None

    def _map_contiguity(self, selections):
        from abjad.tools import selectortools
        if self.allow_discontiguity:
            return selections
        selector = selectortools.ContiguitySelectorCallback()
        result = []
        for selection in selections:
            parts = selector(selection)
            result.extend(parts)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def allow_discontiguity(self):
        r'''Is true when selector allow discontiguity.

        Otherwise selector further groups by contiguity.

        Set to true, false or none.

        Defaults to none.

        Returns true, false or none.
        '''
        return self._allow_discontiguity



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ChordInversion___eq___01():

    chord_inversion = tonalanalysistools.ChordInversion(0)
    u = tonalanalysistools.ChordInversion(0)
    voice = tonalanalysistools.ChordInversion(1)

    assert      chord_inversion == chord_inversion
    assert      chord_inversion == u
    assert not chord_inversion == voice

    assert      u == chord_inversion
    assert      u == u
    assert not u == voice

    assert not voice == chord_inversion
    assert not voice == u
    assert      voice == voice



# -*- coding: utf-8 -*-
from abjad.tools import patterntools
from abjad.tools.topleveltools import new


def silence_except(indices=None):
    r'''Makes silence mask that matches all indices except `indices`.

    ..  container:: example

        Silences divisions except 1 and 2:

        ::

            >>> mask = rhythmmakertools.silence_except([1, 2])

        ::

            >>> f(mask)
            rhythmmakertools.SilenceMask(
                pattern=patterntools.Pattern(
                    indices=[1, 2],
                    inverted=True,
                    ),
                )

        ::

            >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
            ...     division_masks=[mask],
            ...     )
            >>> divisions = [(7, 16), (3, 8), (7, 16), (3, 8)]
            >>> selections = rhythm_maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     selections,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> staff = lilypond_file[Staff]
            >>> f(lilypond_file[Staff])
            \new RhythmicStaff {
                {
                    \time 7/16
                    r4..
                }
                {
                    \time 3/8
                    c'4.
                }
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    r4.
                }
            }

    ..  container:: example

        Silences divisions except -1 and -2:

        ::

            >>> mask = rhythmmakertools.silence_except([-1, -2])

        ::

            >>> f(mask)
            rhythmmakertools.SilenceMask(
                pattern=patterntools.Pattern(
                    indices=[-1, -2],
                    inverted=True,
                    ),
                )

        ::

            >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
            ...     division_masks=[
            ...         mask,
            ...         ],
            ...     )
            >>> divisions = [(7, 16), (3, 8), (7, 16), (3, 8)]
            >>> selections = rhythm_maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     selections,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> f(lilypond_file[Staff])
            \new RhythmicStaff {
                {
                    \time 7/16
                    r4..
                }
                {
                    \time 3/8
                    r4.
                }
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    c'4.
                }
            }

    ..  container:: example

        Works with pattern input:

        ::

            >>> pattern_1 = patterntools.select_all()
            >>> pattern_2 = patterntools.select_first()
            >>> pattern_3 = patterntools.select_last()
            >>> pattern = pattern_1 ^ pattern_2 ^ pattern_3
            >>> mask = rhythmmakertools.silence_except(pattern)

        ::

            >>> f(mask)
            rhythmmakertools.SilenceMask(
                pattern=patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[0],
                            period=1,
                            ),
                        patterntools.Pattern(
                            indices=[0],
                            ),
                        patterntools.Pattern(
                            indices=[-1],
                            ),
                        ),
                    inverted=True,
                    operator='xor',
                    ),
                )

        ::

            >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
            ...     division_masks=[
            ...         mask,
            ...         ],
            ...     )
            >>> divisions = [(7, 16), (3, 8), (7, 16), (3, 8)]
            >>> selections = rhythm_maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     selections,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> f(lilypond_file[Staff])
            \new RhythmicStaff {
                {
                    \time 7/16
                    r4..
                }
                {
                    \time 3/8
                    c'4.
                }
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    r4.
                }
            }

    Equivalent to ``silence(..., inverted=True)``.

    Returns silence mask.
    '''
    from abjad.tools import rhythmmakertools
    indices = indices or []
    prototype = (patterntools.Pattern, patterntools.CompoundPattern)
    if isinstance(indices, prototype):
        pattern = indices
    else:
        pattern = patterntools.Pattern(
            indices=indices,
            )
    pattern = new(pattern, inverted=True)
    return rhythmmakertools.SilenceMask(pattern=pattern)



# -*- coding: utf-8 -*-
import copy
from abjad import *


def test_scoretools_Skip___copy___01():
    r'''Copy skip.
    '''

    skip_1 = scoretools.Skip((1, 4))
    skip_2 = copy.copy(skip_1)

    assert isinstance(skip_1, scoretools.Skip)
    assert isinstance(skip_2, scoretools.Skip)
    assert format(skip_1) == format(skip_2)
    assert skip_1 is not skip_2


def test_scoretools_Skip___copy___02():
    r'''Copy skip with LilyPond multiplier.
    '''

    skip_1 = scoretools.Skip('s4')
    attach(Multiplier(1, 2), skip_1)
    skip_2 = copy.copy(skip_1)

    assert isinstance(skip_1, scoretools.Skip)
    assert isinstance(skip_2, scoretools.Skip)
    assert format(skip_1) == format(skip_2)
    assert skip_1 is not skip_2


def test_scoretools_Skip___copy___03():
    r'''Copy skip with LilyPond grob overrides and LilyPond context settings.
    '''

    skip_1 = scoretools.Skip((1, 4))
    override(skip_1).staff.note_head.color = 'red'
    override(skip_1).accidental.color = 'red'
    set_(skip_1).tuplet_full_length = True
    skip_2 = copy.copy(skip_1)

    assert isinstance(skip_1, scoretools.Skip)
    assert isinstance(skip_2, scoretools.Skip)
    assert format(skip_1) == format(skip_2)
    assert skip_1 is not skip_2



# -*- coding: utf-8 -*-


def all_are_positive_integer_powers_of_two(argument):
    '''Is true when `argument` is an iterable collection of positive integer
    powers of two. Otherwise false.

    ..  container:: example

        ::

            >>> items = [1, 1, 1, 2, 4, 32, 32]
            >>> mathtools.all_are_nonnegative_integer_powers_of_two(items)
            True

        ::

            >>> mathtools.all_are_nonnegative_integer_powers_of_two(17)
            False

    ..  container:: example

        Is true when `argument` is empty:

        ::

            >>> mathtools.all_are_nonnegative_integer_powers_of_two([])
            True

    Returns true or false.
    '''
    from abjad.tools import mathtools
    try:
        return all(
            mathtools.is_positive_integer_power_of_two(_) for _ in argument
            )
    except TypeError:
        return False



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import rhythmtreetools


def test_rhythmtreetools_RhythmTreeNode___call___01():

    rtm = '(1 (1 1 1 1))'
    tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
    result = tree((1, 4))

    assert isinstance(result, (list, selectiontools.Selection))
    assert len(result) == 4
    assert all(isinstance(x, Note) for x in result)
    assert all(x.written_duration == Duration(1, 16) for x in result)


def test_rhythmtreetools_RhythmTreeNode___call___02():

    rtm = '(1 (1 (2 (1 1 1)) 2))'
    tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
    result = tree((1, 4))

    assert isinstance(result, (list, selectiontools.Selection))
    assert len(result) == 1
    assert isinstance(result[0], scoretools.FixedDurationTuplet)
    assert format(result[0]) == stringtools.normalize(
        r'''
        \times 4/5 {
            c'16
            \times 2/3 {
                c'16
                c'16
                c'16
            }
            c'8
        }
        '''
        )


def test_rhythmtreetools_RhythmTreeNode___call___03():

    rtm = '(1 (1 (2 (1 (2 (1 1)) 1)) 2))'
    tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
    result = tree((1, 4))

    assert format(result[0]) == stringtools.normalize(
        r'''
        \times 4/5 {
            c'16
            c'32
            c'32
            c'32
            c'32
            c'8
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import lilypondfiletools
from abjad.tools import markuptools
from abjad.tools.topleveltools import override


def configure_lilypond_file(lilypond_file):
    r'''Configures LilyPond file.
    '''

    lilypond_file._global_staff_size = 8

    context_block = lilypondfiletools.ContextBlock(
        source_context_name=r'Staff \RemoveEmptyStaves',
        )
    override(context_block).vertical_axis_group.remove_first = True
    lilypond_file.layout_block.items.append(context_block)

    slash_separator = indicatortools.LilyPondCommand('slashSeparator')
    lilypond_file.paper_block.system_separator_markup = slash_separator

    bottom_margin = lilypondfiletools.LilyPondDimension(0.5, 'in')
    lilypond_file.paper_block.bottom_margin = bottom_margin

    top_margin = lilypondfiletools.LilyPondDimension(0.5, 'in')
    lilypond_file.paper_block.top_margin = top_margin

    left_margin = lilypondfiletools.LilyPondDimension(0.75, 'in')
    lilypond_file.paper_block.left_margin = left_margin

    right_margin = lilypondfiletools.LilyPondDimension(0.5, 'in')
    lilypond_file.paper_block.right_margin = right_margin

    paper_width = lilypondfiletools.LilyPondDimension(5.25, 'in')
    lilypond_file.paper_block.paper_width = paper_width

    paper_height = lilypondfiletools.LilyPondDimension(7.25, 'in')
    lilypond_file.paper_block.paper_height = paper_height

    lilypond_file.header_block.composer = markuptools.Markup('Arvo Pärt')
    title = 'Cantus in Memory of Benjamin Britten (1980)'
    lilypond_file.header_block.title = markuptools.Markup(title)


# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedInterval___add___01():

    mdi1 = pitchtools.NamedInterval('major', 2)
    mdi2 = pitchtools.NamedInterval('major', 3)
    new = mdi1 + mdi2

    assert new == pitchtools.NamedInterval('augmented', 4)



# -*- coding: utf-8 -*-


def is_integer_equivalent_n_tuple(argument, n):
    r'''Is true when `argument` is a tuple of `n` integer-equivalent items.
    Otherwise false.

    ..  container:: example

        ::

            >>> tuple_ = (2.0, '3', Fraction(4, 1))
            >>> mathtools.is_integer_equivalent_n_tuple(tuple_, 3)
            True

        ::

            >>> tuple_ = (2.5, '3', Fraction(4, 1))
            >>> mathtools.is_integer_equivalent_n_tuple(tuple_, 3)
            False

    Returns true or false.
    '''
    from abjad.tools import mathtools
    return (
        isinstance(argument, tuple) and
        len(argument) == n and
        all(mathtools.is_integer_equivalent(_) for _ in argument)
        )
